package com.tzld.piaoquan.wecom.job;

import com.alibaba.fastjson.JSONObject;
import com.aliyun.odps.data.Record;
import com.tzld.piaoquan.wecom.common.constant.MiniprogramConstant;
import com.tzld.piaoquan.wecom.dao.mapper.MessageAttachmentMapper;
import com.tzld.piaoquan.wecom.dao.mapper.UserMapper;
import com.tzld.piaoquan.wecom.model.bo.PushMessage;
import com.tzld.piaoquan.wecom.model.po.MessageAttachment;
import com.tzld.piaoquan.wecom.model.po.MessageAttachmentExample;
import com.tzld.piaoquan.wecom.model.po.User;
import com.tzld.piaoquan.wecom.model.po.UserExample;
import com.tzld.piaoquan.wecom.utils.DateUtil;
import com.tzld.piaoquan.wecom.utils.OdpsUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

@Component
public class WeComMessageDataJob {

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private MessageAttachmentMapper messageAttachmentMapper;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    private static final int sum = 3;

    List<PushMessage> goodList = new ArrayList<>();
    List<Long> saveList = new ArrayList<>();

    //初始化操作
    void init() {
        String sql = String.format("SELECT * FROM loghubods.history_good_video_can_push_user_list where dt = %s;", DateUtil.getBeforeDayDateString());
        List<Record> recordList = OdpsUtil.getOdpsData(sql);
        if (CollectionUtils.isEmpty(recordList)) {
            return;
        }
        List<PushMessage> list = new ArrayList<>();
        for (Record record : recordList) {
            PushMessage pushMessage = new PushMessage();
            Long videoId = Long.parseLong((String) record.get(0));
            Set<Long> userIds = new HashSet<>(JSONObject.parseArray((String) record.get(1), Long.class));
            pushMessage.setVideoId(videoId);
            pushMessage.setUserIds(userIds);
            list.add(pushMessage);
        }
        goodList = list;
        List<Long> saveVideoIds = Objects.requireNonNull(redisTemplate.opsForList().range(MiniprogramConstant.GUARANTEED_MINIPROGRAM_KEY, 0, 2)).stream().map(o -> (Long) o).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(saveVideoIds)) {
            throw new RuntimeException("保底数据为空");
        }
        MessageAttachmentExample example = new MessageAttachmentExample();
        example.createCriteria().andMiniprogramVideoIdIn(saveVideoIds);
        List<MessageAttachment> messageAttachmentList = messageAttachmentMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(messageAttachmentList) || messageAttachmentList.size() < saveVideoIds.size()) {
            throw new RuntimeException("保底数据获取失败");
        }
        for (MessageAttachment messageAttachment : messageAttachmentList) {
            if (messageAttachment.getUpdateTime().compareTo(DateUtil.getThatDayDate()) < 0) {
                throw new RuntimeException("保底数据不是最新的");
            }
        }
        saveList = saveVideoIds;
    }

    public void assemble(){
        UserExample example = new UserExample();
    }


    public String getPushKey(User user) {
        int n = 0;
        StringBuilder key = new StringBuilder();
        for (PushMessage pushMessage : goodList) {
            if (pushMessage.getUserIds().contains(user.getId())) {
                if (key.length() == 0) {
                    key.append(pushMessage.getVideoId());
                    n++;
                } else {
                    key.append("_");
                    key.append(pushMessage.getVideoId());
                }
                n++;
                if (n > sum) {
                    return key.toString();
                }
            }
        }
        for (Long videoId : saveList) {
            if (key.length() == 0) {
                key.append(videoId);
                n++;
            } else {
                key.append("_");
                key.append(videoId);
            }
            n++;
            if (n > sum) {
                return key.toString();
            }
        }
        return key.toString();
    }
}
